package ila;

import ila.signals.*;

testsuite LogicalLightCtrlTest for LightCtrl(3) { 
    
    test noInput {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <10 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, 10 * Tk>;
        }
    }
    
    test switchStatusOnOnly {
        input {
            switchStatus:   <10 * (SwitchStatus.ON, Tk)>;
            alarmStatus:    <10 * Tk>;
            doorStatus:     <10 * Tk>;
        }
        expect {
            cmd:      <Tk, OnOffCmd.ON, 10 * Tk>;
        }    
    }
    
    test switchStatusOnDoorStatusIgnored {
        input {
            switchStatus:   <10 * (SwitchStatus.ON, Tk)>;
            alarmStatus:    <10 * Tk>;
            doorStatus :    <3 * (DoorStatus.OPENED, Tk), 4 * (DoorStatus.CLOSED, Tk), 3 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            cmd:      <Tk, OnOffCmd.ON, 10 * Tk>;
        }    
    }
    
    test switchStatusOffOnly {
        input {
            switchStatus:   <10 * (SwitchStatus.OFF, Tk)>;
            alarmStatus:    <10 * Tk>;
            doorStatus:     <10 * Tk>;
        }
        expect {
            cmd:      <Tk, OnOffCmd.OFF, 10 * Tk>;
        }    
    }

    test switchStatusOffDoorStatusIgnored {
        input {
            switchStatus:   <10 * (SwitchStatus.OFF, Tk)>;
            alarmStatus:    <10 * Tk>;
            doorStatus :    <3 * (DoorStatus.OPENED, Tk), 4 * (DoorStatus.CLOSED, Tk), 3 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            cmd:      <Tk, OnOffCmd.OFF, 10 * Tk>;
        }    
    }
        
    test switchStatusDoorDepOnly {
        input {
            switchStatus:   <10 * (SwitchStatus.DOOR_DEPENDENT, Tk)>;
            alarmStatus:    <10 * Tk>;
            doorStatus:     <10 * Tk>;
        }
        expect {
            cmd:      <Tk, OnOffCmd.OFF, 10 * Tk>;
        }    
    }
    
    test alarmOffOnly {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * (AlarmStatus.OFF, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <10 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, 10 * Tk>;
        }
    }
    
    test alarmOnOnly {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <19 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <19 * (AlarmStatus.ON, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <19 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 5 * Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 3 * Tk>;
        }
    }
    
    test alarmOnThenOff {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <19 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <4 * (AlarmStatus.ON, Tk), 15 * (AlarmStatus.OFF, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <19 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 3 * Tk, OnOffCmd.OFF, 15 * Tk>;
        }
    }    

    test alarmOnThenOffWhileSwitchStatusIsOn {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <19 * (SwitchStatus.ON, Tk)>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <7 * (AlarmStatus.ON, Tk), 15 * (AlarmStatus.OFF, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <19 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.ON, Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 1 * Tk, OnOffCmd.ON, 12 * Tk>;
        }
    }   
        
    test alarmOnSwitchStatusIgnored {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <10 * (SwitchStatus.DOOR_DEPENDENT, Tk), 5 * (SwitchStatus.ON, Tk), 4 * (SwitchStatus.OFF, Tk)>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <19 * (AlarmStatus.ON, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <19 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 5 * Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 3 * Tk>;
        }
    }
    
    test alarmOnDoorStatusIgnored {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <19 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <19 * (AlarmStatus.ON, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <6 * (DoorStatus.OPENED, Tk), 6 * (DoorStatus.CLOSED, Tk), 7 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 5 * Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 3 * Tk>;
        }
    }
    
    test alarmOnDoorStatusAndSwitchStatusIgnored {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <10 * (SwitchStatus.DOOR_DEPENDENT, Tk), 5 * (SwitchStatus.ON, Tk), 4 * (SwitchStatus.OFF, Tk)>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <19 * (AlarmStatus.ON, Tk)>;
            // type: ila.signals.DoorStatus
            doorStatus : <6 * (DoorStatus.OPENED, Tk), 6 * (DoorStatus.CLOSED, Tk), 7 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 5 * Tk, OnOffCmd.ON, 5 * Tk, OnOffCmd.OFF, 3 * Tk>;
        }
    }         
    
    test doorStatusClosedOnly {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <DoorStatus.CLOSED, 10 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, 10 * Tk>;
        }
    }

    test doorStatusOpenedOnly {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <DoorStatus.OPENED, 10 * Tk>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, 10 * Tk>;
        }
    }
    
    test doorStatusClosedDoorDependent {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <SwitchStatus.DOOR_DEPENDENT, 10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <10 * (DoorStatus.CLOSED, Tk)>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, 10 * Tk>;
        }
    }
    test doorStatusOpenedDoorDependent {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <SwitchStatus.DOOR_DEPENDENT, 10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <10 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 9 * Tk>;
        }
    }

    test doorStatusAlternatingDoorDependent {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <SwitchStatus.DOOR_DEPENDENT, 10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <3 * (DoorStatus.OPENED, Tk), 4 * (DoorStatus.CLOSED, Tk), 3 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 6 * Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 2 * Tk>;
        }
    }    

    test doorStatusAlternatingDoorDependentCancelFadeOutTimer {
        input {
            // type: ila.signals.SwitchStatus
            switchStatus : <SwitchStatus.DOOR_DEPENDENT, 10 * Tk>;
            // type: ila.signals.AlarmStatus
            alarmStatus : <10 * Tk>;
            // type: ila.signals.DoorStatus
            doorStatus : <3 * (DoorStatus.OPENED, Tk), 2 * (DoorStatus.CLOSED, Tk), 5 * (DoorStatus.OPENED, Tk)>;
        }
        expect {
            // type: ila.signals.OnOffCmd
            cmd : <Tk, OnOffCmd.OFF, Tk, OnOffCmd.ON, 9 * Tk>;
        }
    }      
}
